
/* Standby Sample 
 * Uses:
 * 1. Standby shortcut.
 * 2. Use it with task scheduler to customise when you want the computer
 * to go to sleep. eg. Between 10pm and 8am after 25 mins of idle time.
 * 3. Give the user the option of putting the computer to sleep
 * at the end of a very long operation by your program. eg. defrag. */

#include <windows.h>

/* Define this to 'printf' and compile as a console program for debug mode */
#define PRINTF //

/* We load SetSystemPowerState dynamically in case it is not in the libraries */
/* Prototype: SetSystemPowerState(BOOL fSuspend, BOOL fForce); */
typedef BOOL   (CALLBACK* LP_SETSYSTEMPOWERSTATE)(BOOL, BOOL);

/* Prototypes */
BOOL EnableProcessPrivilege(LPCTSTR lpPrivilege);



/* --------------------------------------------------------------- */
int main() {

	/* var to store the function address */
	LP_SETSYSTEMPOWERSTATE lpSetSystemPowerState = NULL; 

	/* We need the SE_SHUTDOWN_NAME privilege on NT to put the system in standby
	 * No error checking because we can try even if we failed
	 * to get the neccessary privilege */
	EnableProcessPrivilege(SE_SHUTDOWN_NAME);

	/* Get procedure address so we can call it */
	lpSetSystemPowerState = (LP_SETSYSTEMPOWERSTATE) 
		GetProcAddress(GetModuleHandle(TEXT("kernel32")),
		"SetSystemPowerState");

	if (lpSetSystemPowerState == NULL) {
		PRINTF("GetProcAddress error: %u\n", GetLastError() ); 
		return -1;
	}


	/* Standby system */
	if ( !lpSetSystemPowerState(
		TRUE,     // true for suspend, false for hibernate
		FALSE) )  // true to force action, false to ask other drivers/programs for ok
	{
		PRINTF("SetSystemPowerState error: %u\n", GetLastError() ); 
		return -1;
	}

	/* At this point the computer has just woken up. Therefore we could
	 * record, or return, the wakeup time if needed */

	return 0; //success!
}



/* --------------------------------------------------------------- */
BOOL EnableProcessPrivilege(LPCTSTR lpPrivilege) {

	/* This function enables a privilege for the current process */
	/* lpPrivilege can be either a privilege constant eg. SE_SHUTDOWN_NAME
	 * which is DEFINEd in the windows headers to the string 
	 * "SeShutdownPrivilege" or the string can be used directly. */

	TOKEN_PRIVILEGES tp;
	HANDLE hToken;
	OSVERSIONINFO osvi;


	/* Check that we are a Win NT system before proceeding */
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx (&osvi);
	if (osvi.dwPlatformId != VER_PLATFORM_WIN32_NT)
	{
		PRINTF("Error: System is not NT based");
		return FALSE;
	}


	/* Get the local id of our desired privilege */
	if ( !LookupPrivilegeValue( 
		NULL,                        // lookup privilege on local system
		lpPrivilege,                 // privilege to lookup 
		&tp.Privileges[0].Luid ) )   // receives LUID of privilege
	{
		PRINTF("LookupPrivilegeValue error: %u\n", GetLastError() ); 
		return FALSE; 
	}


	/* Fill in TOKEN_PRIVILEGES structure */
	tp.PrivilegeCount = 1;
	tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;


	/* Get the access token for the current process */
	if ( !OpenProcessToken(
		GetCurrentProcess(),                    // open token for current process
		TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY,  // desired access
		&hToken ) )                             // receives token
	{
		PRINTF("OpenProcessToken error: %u\n", GetLastError() ); 
		return FALSE; 
	}


	/* Enable the privilege */
	if ( !AdjustTokenPrivileges(
	       hToken,         // access token
	       FALSE,          // don't disable all privileges
	       &tp,            // TOKEN_PRIVILEGES struct
	       0,              // sizeof prevState buffer
	       (PTOKEN_PRIVILEGES) NULL, // prevState buffer
	       (PDWORD) NULL)            // receives required size of prevState buffer

	       /* AdjustTokenPrivileges returns success even if not all
	        * desired privileges could be enabled so check GetLastError */
	       || GetLastError() != ERROR_SUCCESS )
	{ 
		PRINTF("AdjustTokenPrivileges error: %u\n", GetLastError() ); 
		CloseHandle(hToken);
		return FALSE; 
	} 


	/* Close the process access token */
	CloseHandle(hToken);


	return TRUE; //success
}


